"use strict";
/**
 * @file Ce fichier contient la classe de l'API du LDAP qui gère les opérations sur les groupes. Elle est destinée à être exportée pour être utilisée par les resolvers.
 * @author hawkspar
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var config_1 = require("../internal/config");
var basics_1 = require("../internal/basics");
var tools_1 = require("../internal/tools");
//------------------------------------------------------------------------------------------------------------------------
// Classes à exporter TBT
//------------------------------------------------------------------------------------------------------------------------
var Group = /** @class */ (function () {
    /**
     * @memberof LDAP
     * @class Group
     * @classdesc Cette classe est une des deux classes exportables permettant de faire des opérations sur les groupes.
     * @summary Constructeur vide.
     */
    function Group() {
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui renvoit toutes les infos relatives à un groupe particulier.
     * @desc Cette fonction utilise {@link Tools.peek} avec l'interface {@link groupData}. Elle rajoute les individus
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(groupData)} Informations recueillies ; renvoie une liste de dictionnaire avec le profil complet du groupe au format {@link groupData}.
     * @static
     * @async
     */
    Group.peek = function (gid) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                try {
                    return [2 /*return*/, tools_1.Tools.peek("group", gid, config_1.groupData)];
                }
                catch (err) {
                    throw "Erreur lors d'une recherche d'informations sur un groupe.";
                }
                return [2 /*return*/];
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui retrouve le groupe qui ressemblent à l'input et qui correspond au type fourni. Etape 0 vers un vrai TOL (Trombino On Line) pour les groupes.
     * @desc Cette fonction utilise {@link Tools.search}.
     * @arg {groupData} input - String entré par l'utilisateur qui ressemble au nom du groupe.
     * @return {Promise(string[])} Liste des gid dont le nom ressemble à l'input.
     * @static
     * @async
    */
    Group.search = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                try {
                    return [2 /*return*/, tools_1.Tools.search("group", data)];
                }
                catch (err) {
                    throw "Erreur lors de la recherche approximative d'un groupe.";
                }
                return [2 /*return*/];
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de rajouter un administrateur à un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}. Le nouvel administrateur ne devient pas membre ou porte-parole du groupe pour autant !
     * @arg {string} uid - Identifiant du membre futur admin
     * @arg {string} gid - Identifiant du groupe
     * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.addAdmin = function (uid, gid) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, tools_1.Tools.add(uid, gid, "admins")];
                case 1: return [2 /*return*/, _a.sent()];
            }
        }); });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de supprimer un administrateur.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.remove}.
     * Elle ne remonte pas les échelons, car cela permettrait à un admin d'un petit groupe de supprimer un admin d'un grand.
     * @arg {string} uid - Identifiant de l'admin à dégrader, supposé membre
     * @arg {string} gid - Identifiant du groupe
     * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.remAdmin = function (uid, gid) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/, tools_1.Tools.remove(uid, gid, "admins")];
        }); });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de rajouter un porte-parole à un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}. Elle ne rajoute pas l'utilisateur au groupe.
     * @arg {string} uid - Identifiant du membre futur porte-parole
     * @arg {string} gid - Identifiant du groupe
     * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.addSpeaker = function (uid, gid) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, tools_1.Tools.add(uid, gid, "speakers")];
                case 1: return [2 /*return*/, _a.sent()];
            }
        }); });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de rétrograder un membre du stade de porte-parole d'un groupe au stade d'utilisateur.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.remove}. Elle dégrade aussi d'un éventuel statut d'administrateur.
     * @arg {string} uid - Identifiant de l'admin à dégrader (pas supposé valide)
     * @arg {string} gid - Identifiant du groupe
     * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.remSpeaker = function (uid, gid) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, tools_1.Tools.remove(uid, gid, "speakers")];
                case 1: return [2 /*return*/, _a.sent()];
            }
        }); });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui permet d'ajouter un utilisateur à un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}.
     * @arg {string} uid - Identifiant de l'utilisateur à ajouter
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.addMember = function (uid, gid) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/, tools_1.Tools.add(uid, gid, "members")];
        }); });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de supprimer un membre existant d'un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}.
     * Cette fonction supprime tous les droits de l'utilisateur sur le groupe, mais aussi sur les groupes sources si son statut de membre était hérité.
     * @arg {string} uid - Identifiant de l'ex-membre (pas supposé valide)
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.remMember = function (uid, gid) {
        return __awaiter(this, void 0, void 0, function () {
            var stack, res, visited, cur_id, _a, _b, _c, _d, _e;
            return __generator(this, function (_f) {
                switch (_f.label) {
                    case 0: return [4 /*yield*/, tools_1.Tools.remove(uid, gid, "members")];
                    case 1: return [2 /*return*/, _f.sent()];
                    case 2:
                        if (!(stack.length > 0)) return [3 /*break*/, 11];
                        cur_id = stack.pop();
                        if (!(visited[cur_id] == undefined)) return [3 /*break*/, 10];
                        visited[cur_id] = true;
                        _c = res;
                        if (!_c) return [3 /*break*/, 4];
                        return [4 /*yield*/, tools_1.Tools.remove(uid, cur_id, "admins")];
                    case 3:
                        _c = (_f.sent());
                        _f.label = 4;
                    case 4:
                        _b = _c;
                        if (!_b) return [3 /*break*/, 6];
                        return [4 /*yield*/, tools_1.Tools.remove(uid, cur_id, "speakers")];
                    case 5:
                        _b = (_f.sent());
                        _f.label = 6;
                    case 6:
                        _a = _b;
                        if (!_a) return [3 /*break*/, 8];
                        return [4 /*yield*/, tools_1.Tools.remove(uid, cur_id, "members")];
                    case 7:
                        _a = (_f.sent());
                        _f.label = 8;
                    case 8:
                        res = _a;
                        _e = (_d = stack).concat;
                        return [4 /*yield*/, basics_1.Basics.searchSingle("group", config_1.ldapConfig.group.childs, cur_id)];
                    case 9:
                        _e.apply(_d, [_f.sent()]);
                        _f.label = 10;
                    case 10: return [3 /*break*/, 2];
                    case 11: return [2 /*return*/, res];
                }
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui permet d'ajouter un sympathisant à un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}.
     * @arg {string} uid - Identifiant de l'utilisateur à ajouter
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.addFollower = function (uid, gid) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/, tools_1.Tools.add(uid, gid, "followers")];
        }); });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de supprimer un sympathisant d'un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}.
     * @arg {string} uid - Identifiant de l'ex-sympathisant (pas supposé valide)
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.remFollower = function (uid, gid) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/, tools_1.Tools.remove(uid, gid, "followers")];
        }); });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui créé un nouveau groupe dans le LDAP.
     * @desc Cette fonction fait une utilisation massive d'eval pour anonymiser son code ; c'est mal et cela suppose que beaucoup de soins ont été pris lors de
     * l'escape de ses paramètres. Appelle {@link LDAP.add} et {@link LDAP.change}, mais aussi {@link Tools.add}
     * pour gérer les groupes du nouvel utilisateur. Cettte application permet de rajouter des utilisateurs à toutes les catégories du groupe.
     * @arg {groupData} data - Dictionnaire des informations utilisateurs (voir détail des champs dans ldapConfig.json)
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.create = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var vals, gid, key_att, vals2, _loop_1, cat;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        vals = {};
                        // gid de base généré à partir du nom standardisé, pas à partir de l'entrée 'gid' !
                        try {
                            tools_1.Tools.generateReadableId(data['name']).then(function (id) {
                                vals[config_1.ldapConfig.key_id] = id;
                                vals[config_1.ldapConfig.group['name']] = id;
                            });
                        }
                        catch (err) {
                            throw "Erreur lors de la génération d'un hruid pour créer un nouveau groupe.";
                        }
                        gid = vals[config_1.ldapConfig.key_id];
                        // Ecriture de toutes les valeurs directement inscrites dans le LDAP
                        for (key_att in data) {
                            vals[config_1.ldapConfig.group[key_att]] = data[key_att];
                        }
                        ;
                        return [4 /*yield*/, basics_1.Basics.add("group", vals)];
                    case 1:
                        // Appel à la fonction de base
                        if (!(_a.sent())) {
                            throw "Erreur lors de la création d'une nouvelle feuille dans l'arbre des groupes.";
                        }
                        vals2 = {};
                        // ?!
                        vals2[config_1.ldapConfig.group['password']] = "{CRYPT}" + data['password'];
                        // Génération id aléatoire et test contre le LDAP
                        try {
                            tools_1.Tools.generateId(config_1.ldapConfig.group["idNumber"], "group").then(function (id) { vals2[config_1.ldapConfig.group['idNumber']] = id; });
                        }
                        catch (err) {
                            throw "Erreur lors de la génération d'un id numérique pour créer un nouveau groupe.";
                        }
                        // Code root
                        vals2[config_1.ldapConfig.group['cleanFullName']] = data['name'].replace(':', ';').toLowerCase().normalize('UFD');
                        return [4 /*yield*/, basics_1.Basics.change("group", gid, "add", vals2)];
                    case 2:
                        // Inscription des valeurs calculées par effet de bord
                        if (!(_a.sent())) {
                            throw "Erreur lors de l'ajout des valeurs intelligentes du nouveau groupe.";
                        }
                        ["posixGroup", "brGroup"].forEach(function (cst) {
                            var vals3 = {};
                            vals3[config_1.ldapConfig.group['classes']] = cst;
                            basics_1.Basics.change("group", gid, "add", vals3).then(function (res) {
                                if (!res) {
                                    throw "Erreur lors de l'ajout des valeurs constantes du nouveau groupe.";
                                }
                            });
                        });
                        _loop_1 = function (cat) {
                            data[cat].forEach(function (uid) {
                                tools_1.Tools.add(uid, gid, cat).then(function (res) {
                                    if (!res) {
                                        throw "Erreur de l'ajout d'un membre au nouveau groupe.";
                                    }
                                });
                            });
                        };
                        // Utilisation des fonctions adaptées pour assurer la cohérence de l'ensemble
                        for (cat in config_1.categories) {
                            _loop_1(cat);
                        }
                        return [2 /*return*/, true];
                }
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui supprime un groupe du LDAP.
     * @desc Cette fonction commence par gérer les groupes du membre puis le supprime entièrement. A modifier une fois que le LDAP incluerait les groupes administres par une utilisateur.
     * Appelle {@link LDAP.clear} bien sûr, mais aussi {@link Tools.remove} pour gérer les groupes de l'utilisateur sortant.
     * @arg {string} gid - gid de la victime
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group["delete"] = function (gid) {
        return __awaiter(this, void 0, void 0, function () {
            var profil, _loop_2, _i, categories_1, cat, err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 3, , 4]);
                        return [4 /*yield*/, Group.peek(gid)];
                    case 1:
                        profil = _a.sent();
                        _loop_2 = function (cat) {
                            profil[config_1.ldapConfig.group[cat]].forEach(function quickPartRemUser(uid) {
                                return __awaiter(this, void 0, void 0, function () {
                                    var lg;
                                    return __generator(this, function (_a) {
                                        switch (_a.label) {
                                            case 0: return [4 /*yield*/, tools_1.Tools.get(uid, "user", cat)];
                                            case 1:
                                                lg = _a.sent();
                                                if (!lg.includes(gid)) return [3 /*break*/, 3];
                                                return [4 /*yield*/, basics_1.Basics.change("user", uid, "del", config_1.ldapConfig.user[cat])];
                                            case 2:
                                                // Supprime tous les groupes
                                                if (!(_a.sent())) {
                                                    throw "Erreur lors de la suppression de tous les groupes du membre.";
                                                }
                                                // Les rajoute un par un, sauf pour le groupe supprimé
                                                lg.forEach(function (id) {
                                                    if (id != gid) {
                                                        tools_1.Tools.add(uid, id, cat).then(function (res) {
                                                            if (!res) {
                                                                throw "Erreur lors du ré-ajout des autres groupes";
                                                            }
                                                        });
                                                    }
                                                });
                                                _a.label = 3;
                                            case 3: return [2 /*return*/];
                                        }
                                    });
                                });
                            });
                        };
                        for (_i = 0, categories_1 = config_1.categories; _i < categories_1.length; _i++) {
                            cat = categories_1[_i];
                            _loop_2(cat);
                        }
                        return [4 /*yield*/, basics_1.Basics.clear("group", gid)];
                    case 2:
                        // Elimination
                        if (!(_a.sent())) {
                            throw "Erreur lors de la suppression de la feuille dans l'arbre des groupes.";
                        }
                        return [2 /*return*/, true];
                    case 3:
                        err_1 = _a.sent();
                        throw "Erreur lors de l'obtention du profil d'un groupe pour le supprimer.";
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui édite un groupe existant dans le LDAP. Sans influence sur ses membres ou admins.
     * @desc Appelle {@link Tools.edit}.
     * @arg {groupData} data - Dictionnaire des informations du groupe.
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    Group.edit = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                try {
                    return [2 /*return*/, tools_1.Tools.edit("group", data)];
                }
                catch (err) {
                    throw "Erreur lors de la modification d'un groupe.";
                }
                return [2 /*return*/];
            });
        });
    };
    return Group;
}());
exports.Group = Group;
